@@ -25,19 +25,18 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010-2011 The Orca Team"
 __license__   = "LGPL"
 
-import pyatspi
-
 from . import cmdnames
 from . import debug
+from . import focus_manager
 from . import guilabels
 from . import input_event
 from . import keybindings
 from . import messages
-from . import orca_state
+from . import script_manager
 from . import settings
 from . import settings_manager
-
-_settingsManager = settings_manager.getManager()
+from .ax_object import AXObject
+from .ax_utilities import AXUtilities
 
 #############################################################################
 #                                                                           #
@@ -268,7 +267,7 @@ class ConversationList:
         #
         try:
             self.conversations.remove(conversation)
-        except:
+        except Exception:
             return False
         else:
             return True
@@ -289,7 +288,7 @@ class Chat:
 
         Arguments:
         - script: the script with which this instance is associated.
-        - buddyListAncestries: a list of lists of pyatspi roles beginning
+        - buddyListAncestries: a list of lists of roles beginning
           with the object serving as the actual buddy list (e.g.
           ROLE_TREE_TABLE) and ending with the top level object (e.g.
           ROLE_FRAME).
@@ -319,8 +318,6 @@ class Chat:
         self.messageListLength = len(self.messageKeys)
         self._conversationList = ConversationList(self.messageListLength)
 
-        # To make pylint happy.
-        #
         self.focusedChannelRadioButton = None
         self.allChannelsRadioButton = None
         self.allMessagesRadioButton = None
@@ -404,19 +401,19 @@ class Chat:
         grid.set_border_width(12)
 
         label = guilabels.CHAT_SPEAK_ROOM_NAME
-        value = _settingsManager.getSetting('chatSpeakRoomName')
+        value = settings_manager.getManager().getSetting('chatSpeakRoomName')
         self.speakNameCheckButton = Gtk.CheckButton.new_with_mnemonic(label)
         self.speakNameCheckButton.set_active(value)
         grid.attach(self.speakNameCheckButton, 0, 0, 1, 1)
 
         label = guilabels.CHAT_ANNOUNCE_BUDDY_TYPING
-        value = _settingsManager.getSetting('chatAnnounceBuddyTyping')
+        value = settings_manager.getManager().getSetting('chatAnnounceBuddyTyping')
         self.buddyTypingCheckButton = Gtk.CheckButton.new_with_mnemonic(label)
         self.buddyTypingCheckButton.set_active(value)
         grid.attach(self.buddyTypingCheckButton, 0, 1, 1, 1)
 
         label = guilabels.CHAT_SEPARATE_MESSAGE_HISTORIES
-        value = _settingsManager.getSetting('chatRoomHistories')
+        value = settings_manager.getManager().getSetting('chatRoomHistories')
         self.chatRoomHistoriesCheckButton = \
             Gtk.CheckButton.new_with_mnemonic(label)
         self.chatRoomHistoriesCheckButton.set_active(value)
@@ -424,7 +421,7 @@ class Chat:
 
         messagesFrame = Gtk.Frame()
         grid.attach(messagesFrame, 0, 3, 1, 1)
-        label = Gtk.Label("<b>%s</b>" % guilabels.CHAT_SPEAK_MESSAGES_FROM)
+        label = Gtk.Label(f"<b>{guilabels.CHAT_SPEAK_MESSAGES_FROM}</b>")
         label.set_use_markup(True)
         messagesFrame.set_label_widget(label)
 
@@ -434,7 +431,7 @@ class Chat:
         messagesGrid = Gtk.Grid()
         messagesAlignment.add(messagesGrid)
 
-        value = _settingsManager.getSetting('chatMessageVerbosity')
+        value = settings_manager.getManager().getSetting('chatMessageVerbosity')
 
         label = guilabels.CHAT_SPEAK_MESSAGES_ALL
         rb1 = Gtk.RadioButton.new_with_mnemonic(None, label)
@@ -450,7 +447,7 @@ class Chat:
         messagesGrid.attach(self.focusedChannelRadioButton, 0, 1, 1, 1)
 
         label = guilabels.CHAT_SPEAK_MESSAGES_ALL_IF_FOCUSED % \
-            self._script.app.name
+            AXObject.get_name(self._script.app)
         rb3 = Gtk.RadioButton.new_with_mnemonic(None, label)
         rb3.join_group(rb1)
         rb3.set_active(value == settings.CHAT_SPEAK_ALL_IF_FOCUSED)
@@ -494,8 +491,8 @@ class Chat:
         """
 
         line = messages.CHAT_ROOM_NAME_PREFIX_ON
-        speakRoomName = _settingsManager.getSetting('chatSpeakRoomName')
-        _settingsManager.setSetting('chatSpeakRoomName', not speakRoomName)
+        speakRoomName = settings_manager.getManager().getSetting('chatSpeakRoomName')
+        settings_manager.getManager().setSetting('chatSpeakRoomName', not speakRoomName)
         if speakRoomName:
             line = messages.CHAT_ROOM_NAME_PREFIX_OFF
         self._script.presentMessage(line)
@@ -511,8 +508,8 @@ class Chat:
         """
 
         line = messages.CHAT_BUDDY_TYPING_ON
-        announceTyping = _settingsManager.getSetting('chatAnnounceBuddyTyping')
-        _settingsManager.setSetting(
+        announceTyping = settings_manager.getManager().getSetting('chatAnnounceBuddyTyping')
+        settings_manager.getManager().setSetting(
             'chatAnnounceBuddyTyping', not announceTyping)
         if announceTyping:
             line = messages.CHAT_BUDDY_TYPING_OFF
@@ -529,8 +526,8 @@ class Chat:
         """
 
         line = messages.CHAT_SEPARATE_HISTORIES_ON
-        roomHistories = _settingsManager.getSetting('chatRoomHistories')
-        _settingsManager.setSetting('chatRoomHistories', not roomHistories)
+        roomHistories = settings_manager.getManager().getSetting('chatRoomHistories')
+        settings_manager.getManager().setSetting('chatRoomHistories', not roomHistories)
         if roomHistories:
             line = messages.CHAT_SEPARATE_HISTORIES_OFF
         self._script.presentMessage(line)
@@ -551,14 +548,14 @@ class Chat:
 
         try:
             index = self.messageKeys.index(inputEvent.event_string)
-        except:
+        except Exception:
             pass
 
         messageNumber = self.messageListLength - (index + 1)
         message, chatRoomName = None, None
 
-        if _settingsManager.getSetting('chatRoomHistories'):
-            conversation = self.getConversation(orca_state.locusOfFocus)
+        if settings_manager.getManager().getSetting('chatRoomHistories'):
+            conversation = self.getConversation(focus_manager.getManager().get_locus_of_focus())
             if conversation:
                 message = conversation.getNthMessage(messageNumber)
                 chatRoomName = conversation.name
@@ -583,8 +580,10 @@ class Chat:
         # Only speak/braille the new message if it matches how the user
         # wants chat messages spoken.
         #
-        verbosity = _settingsManager.getAppSetting(self._script.app, 'chatMessageVerbosity')
-        if orca_state.activeScript.name != self._script.name \
+        verbosity = settings_manager.getManager().getAppSetting(
+            self._script.app, 'chatMessageVerbosity')
+        script = script_manager.getManager().getActiveScript()
+        if script.name != self._script.name \
            and verbosity == settings.CHAT_SPEAK_ALL_IF_FOCUSED:
             return
         elif not focused and verbosity == settings.CHAT_SPEAK_FOCUSED_CHANNEL:
@@ -592,7 +591,7 @@ class Chat:
 
         text = ""
         if chatRoomName and \
-           _settingsManager.getAppSetting(self._script.app, 'chatSpeakRoomName'):
+           settings_manager.getManager().getAppSetting(self._script.app, 'chatSpeakRoomName'):
             text = messages.CHAT_MESSAGE_FROM_ROOM % chatRoomName
 
         if not settings.presentChatRoomLast:
@@ -655,12 +654,6 @@ class Chat:
             return True
 
         elif self.isChatRoomMsg(event.source):
-            # We always automatically go back to focus tracking mode when
-            # someone sends us a message.
-            #
-            if self._script.flatReviewContext:
-                self._script.toggleFlatReviewMode()
-
             if self.isNewConversation(event.source):
                 name = self.getChatRoomName(event.source)
                 conversation = Conversation(name, event.source)
@@ -700,7 +693,7 @@ class Chat:
         Returns True if we spoke the change; False otherwise
         """
 
-        if _settingsManager.getSetting('chatAnnounceBuddyTyping'):
+        if settings_manager.getManager().getSetting('chatAnnounceBuddyTyping'):
             conversation = self.getConversation(event.source)
             if conversation and (status != conversation.getTypingStatus()):
                 voice = self._script.speechGenerator.voice(string=status)
@@ -738,12 +731,7 @@ class Chat:
         - obj: the accessible object to examine.
         """
 
-        state = obj.getState()
-        if state.contains(pyatspi.STATE_EDITABLE) \
-           and state.contains(pyatspi.STATE_SINGLE_LINE):
-            return True
-
-        return False
+        return AXUtilities.is_editable(obj) and AXUtilities.is_single_line(obj)
 
     def isBuddyList(self, obj):
         """Returns True if obj is the list of buddies in the buddy list
@@ -756,10 +744,12 @@ class Chat:
         - obj: the accessible being examined
         """
 
-        if obj:
-            for roleList in self._buddyListAncestries:
-                if self._script.utilities.hasMatchingHierarchy(obj, roleList):
-                    return True
+        if obj is None:
+            return False
+
+        for roleList in self._buddyListAncestries:
+            if self._script.utilities.hasMatchingHierarchy(obj, roleList):
+                return True
 
         return False
 
@@ -776,9 +766,8 @@ class Chat:
             return True
 
         for roleList in self._buddyListAncestries:
-            buddyListRole = roleList[0]
-            candidate = self._script.utilities.ancestorWithRole(
-                obj, [buddyListRole], [pyatspi.ROLE_FRAME])
+            role = roleList[0]
+            candidate = AXObject.find_ancestor(obj, lambda x: AXObject.get_role(x) == role)
             if self.isBuddyList(candidate):
                 return True
 
@@ -816,8 +805,8 @@ class Chat:
         # things working. And people should not be in multiple chat
         # rooms with identical names anyway. :-)
         #
-        if obj.getRole() in [pyatspi.ROLE_TEXT, pyatspi.ROLE_ENTRY] \
-           and obj.getState().contains(pyatspi.STATE_EDITABLE):
+        if (AXUtilities.is_text(obj) or AXObject.is_entry(obj)) \
+           and AXUtilities.is_editable(obj):
             name = self.getChatRoomName(obj)
 
         for conversation in self._conversationList.conversations:
@@ -840,13 +829,8 @@ class Chat:
         - obj: the accessible object to examine.
         """
 
-        if obj and obj.getRole() == pyatspi.ROLE_TEXT \
-           and obj.parent.getRole() == pyatspi.ROLE_SCROLL_PANE:
-            state = obj.getState()
-            if not state.contains(pyatspi.STATE_EDITABLE) \
-               and state.contains(pyatspi.STATE_MULTI_LINE):
-                return True
-
+        if AXUtilities.is_text(obj) and AXUtilities.is_scroll_pane(AXObject.get_parent(obj)):
+            return not AXUtilities.is_editable(obj) and AXUtilities.is_multi_line(obj)
         return False
 
     def isFocusedChat(self, obj):
@@ -858,14 +842,14 @@ class Chat:
         - obj: the accessible object to examine.
         """
 
-        if obj and obj.getState().contains(pyatspi.STATE_SHOWING):
+        if AXUtilities.is_showing(obj):
             active = self._script.utilities.topLevelObjectIsActiveAndCurrent(obj)
-            msg = "INFO: %s's window is focused chat: %s" % (obj, active)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["INFO:", obj, "'s window is focused chat:", active]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return active
 
-        msg = "INFO: %s is not focused chat (not showing)" % obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["INFO:", obj, "is not focused chat (not showing)"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return False
 
     def getChatRoomName(self, obj):
@@ -882,16 +866,16 @@ class Chat:
         # that, we'll look at the frame name. Failing that, scripts
         # should override this method. :-)
         #
-        ancestor = self._script.utilities.ancestorWithRole(
-            obj,
-            [pyatspi.ROLE_PAGE_TAB, pyatspi.ROLE_FRAME],
-            [pyatspi.ROLE_APPLICATION])
+        def pred(x):
+            return AXUtilities.is_page_tab(x) or AXUtilities.is_frame(x)
+
+        ancestor = AXObject.find_ancestor(obj, pred)
         name = ""
         try:
             text = self._script.utilities.displayedText(ancestor)
             if text.lower().strip() != self._script.name.lower().strip():
                 name = text
-        except:
+        except Exception:
             pass
 
         # Some applications don't trash their page tab list when there is
@@ -899,14 +883,13 @@ class Chat:
         # the item. Therefore, we'll give it one more shot.
         #
         if not name:
-            ancestor = self._script.utilities.ancestorWithRole(
-                ancestor, [pyatspi.ROLE_FRAME], [pyatspi.ROLE_APPLICATION])
+            ancestor = AXObject.find_ancestor(obj, AXUtilities.is_frame)
             try:
                 text = self._script.utilities.displayedText(ancestor)
                 if text.lower().strip() != self._script.name.lower().strip():
                     name = text
-            except:
-                pass     
+            except Exception:
+                pass
 
         return name
 
@@ -917,7 +900,7 @@ class Chat:
         - event: the accessible event being examined
         """
 
-        if event.source.getRole() != pyatspi.ROLE_TEXT:
+        if not AXUtilities.is_text(event.source):
             return False
 
         lastKey, mods = self._script.utilities.lastKeyAndModifiers()
