@@ -25,21 +25,20 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
-
-import orca.orca as orca
 import orca.cmdnames as cmdnames
 import orca.debug as debug
+import orca.focus_manager as focus_manager
 import orca.input_event as input_event
 import orca.scripts.default as default
 import orca.settings_manager as settings_manager
-import orca.orca_state as orca_state
 import orca.scripts.toolkits.Gecko as Gecko
+from orca.ax_document import AXDocument
+from orca.ax_object import AXObject
+from orca.ax_text import AXText
+from orca.ax_utilities import AXUtilities
 
 from .spellcheck import SpellCheck
 
-_settingsManager = settings_manager.getManager()
-
 ########################################################################
 #                                                                      #
 # The Thunderbird script class.                                        #
@@ -61,10 +60,10 @@ class Script(Gecko.Script):
         #
         self._lastAutoComplete = ""
 
-        if _settingsManager.getSetting('sayAllOnLoad') is None:
-            _settingsManager.setSetting('sayAllOnLoad', False)
-        if _settingsManager.getSetting('pageSummaryOnLoad') is None:
-            _settingsManager.setSetting('pageSummaryOnLoad', False)
+        if settings_manager.getManager().getSetting('sayAllOnLoad') is None:
+            settings_manager.getManager().setSetting('sayAllOnLoad', False)
+        if settings_manager.getManager().getSetting('pageSummaryOnLoad') is None:
+            settings_manager.getManager().setSetting('pageSummaryOnLoad', False)
 
         super().__init__(app)
 
@@ -98,9 +97,9 @@ class Script(Gecko.Script):
         grid = super().getAppPreferencesGUI()
 
         self._sayAllOnLoadCheckButton.set_active(
-            _settingsManager.getSetting('sayAllOnLoad'))
+            settings_manager.getManager().getSetting('sayAllOnLoad'))
         self._pageSummaryOnLoadCheckButton.set_active(
-            _settingsManager.getSetting('pageSummaryOnLoad'))
+            settings_manager.getManager().getSetting('pageSummaryOnLoad'))
 
         spellcheck = self.spellcheck.getAppPreferencesGUI()
         grid.attach(spellcheck, 0, len(grid.get_children()), 1, 1)
@@ -123,7 +122,6 @@ class Script(Gecko.Script):
 
         if self.spellcheck.isSuggestionsItem(newFocus):
             includeLabel = not self.spellcheck.isSuggestionsItem(oldFocus)
-            orca.emitRegionChanged(newFocus)
             self.updateBraille(newFocus)
             self.spellcheck.presentSuggestionListItem(includeLabel=includeLabel)
             return
@@ -132,40 +130,33 @@ class Script(Gecko.Script):
 
     def useFocusMode(self, obj, prevObj=None):
         if self.utilities.isEditableMessage(obj):
-            msg = "THUNDERBIRD: Using focus mode for editable message %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["THUNDERBIRD: Using focus mode for editable message", obj]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return True
 
-        msg = "THUNDERBIRD: %s is not an editable message." % obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["THUNDERBIRD:", obj, "is not an editable message."]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return super().useFocusMode(obj, prevObj)
 
     def enableStickyBrowseMode(self, inputEvent, forceMessage=False):
-        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
+        if self.utilities.isEditableMessage(focus_manager.getManager().get_locus_of_focus()):
             return
 
         super().enableStickyBrowseMode(inputEvent, forceMessage)
 
     def enableStickyFocusMode(self, inputEvent, forceMessage=False):
-        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
+        if self.utilities.isEditableMessage(focus_manager.getManager().get_locus_of_focus()):
             return
 
         super().enableStickyFocusMode(inputEvent, forceMessage)
 
     def togglePresentationMode(self, inputEvent, documentFrame=None):
-        if self._inFocusMode and self.utilities.isEditableMessage(orca_state.locusOfFocus):
+        if self._inFocusMode \
+           and self.utilities.isEditableMessage(focus_manager.getManager().get_locus_of_focus()):
             return
 
         super().togglePresentationMode(inputEvent, documentFrame)
 
-    def useStructuralNavigationModel(self):
-        """Returns True if structural navigation should be enabled here."""
-
-        if self.utilities.isEditableMessage(orca_state.locusOfFocus):
-            return False
-
-        return super().useStructuralNavigationModel()
-
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
@@ -175,8 +166,8 @@ class Script(Gecko.Script):
         self._lastAutoComplete = ""
         obj = event.source
         if self.spellcheck.isAutoFocusEvent(event):
-            orca.setLocusOfFocus(event, event.source, False)
-            self.updateBraille(orca_state.locusOfFocus)
+            focus_manager.getManager().set_locus_of_focus(event, event.source, False)
+            self.updateBraille(event.source)
 
         if not self.utilities.inDocumentContent(obj):
             super().onFocusedChanged(event)
@@ -199,17 +190,13 @@ class Script(Gecko.Script):
 
         obj = event.source
         if self.utilities.isDocument(obj) and not event.detail1:
-            try:
-                role = orca_state.locusOfFocus.getRole()
-                name = orca_state.locusOfFocus.name
-            except:
-                pass
-            else:
-                if role in [pyatspi.ROLE_FRAME, pyatspi.ROLE_PAGE_TAB] and name:
-                    orca.setLocusOfFocus(event, event.source, False)
+            focus = focus_manager.getManager().get_locus_of_focus()
+            if AXObject.get_name(focus) \
+                and (AXUtilities.is_frame(focus) or AXUtilities.is_page_tab(focus)):
+                focus_manager.getManager().set_locus_of_focus(event, event.source, False)
 
             if self.utilities.inDocumentContent():
-                self.speakMessage(obj.name)
+                self.speakMessage(AXObject.get_name(obj))
                 self._presentMessage(obj)
 
     def onCaretMoved(self, event):
@@ -231,9 +218,8 @@ class Script(Gecko.Script):
         if event.source == self.spellcheck.getSuggestionsList():
             return
 
-        parent = event.source.parent
-        if parent and parent.getRole() == pyatspi.ROLE_COMBO_BOX \
-           and not parent.getState().contains(pyatspi.STATE_FOCUSED):
+        parent = AXObject.get_parent(event.source)
+        if AXUtilities.is_combo_box(parent) and not AXUtilities.is_focused(parent):
             return
 
         super().onSelectionChanged(event)
@@ -256,9 +242,8 @@ class Script(Gecko.Script):
         # existent browsery autocompletes for Thunderbird.
 
         if event.detail1 and self.utilities.isMenuWithNoSelectedChild(event.source) \
-           and orca_state.activeWindow == self.utilities.topLevelObject(event.source):
-            self.presentObject(event.source)
-            orca.setLocusOfFocus(event, event.source, False)
+           and self.utilities.topLevelObjectIsActiveWindow(event.source):
+            focus_manager.getManager().set_locus_of_focus(event, event.source, True)
             return
 
         default.Script.onShowingChanged(self, event)
@@ -270,16 +255,8 @@ class Script(Gecko.Script):
         - event: the Event
         """
 
-        obj = event.source
-        parent = obj.parent
-
-        try:
-            role = event.source.getRole()
-            parentRole = parent.getRole()
-        except:
-            return
-
-        if role == pyatspi.ROLE_LABEL and parentRole == pyatspi.ROLE_STATUS_BAR:
+        if AXUtilities.is_label(event.source) \
+           and AXUtilities.is_status_bar(AXObject.get_parent(event.source)):
             return
 
         super().onTextDeleted(event)
@@ -287,29 +264,23 @@ class Script(Gecko.Script):
     def onTextInserted(self, event):
         """Callback for object:text-changed:insert accessibility events."""
 
-        obj = event.source
-        try:
-            role = obj.getRole()
-            parentRole = obj.parent.getRole()
-        except:
+        parent = AXObject.get_parent(event.source)
+        if AXUtilities.is_label(event.source) and AXUtilities.is_status_bar(parent):
             return
 
-        if role == pyatspi.ROLE_LABEL and parentRole == pyatspi.ROLE_STATUS_BAR:
-            return
-
-        if len(event.any_data) > 1 and obj == self.spellcheck.getChangeToEntry():
+        if len(event.any_data) > 1 and event.source == self.spellcheck.getChangeToEntry():
             return
 
         isSystemEvent = event.type.endswith("system")
 
         # Try to stop unwanted chatter when a message is being replied to.
         # See bgo#618484.
-        if isSystemEvent and self.utilities.isEditableMessage(obj):
+        if isSystemEvent and self.utilities.isEditableMessage(event.source):
             return
 
         # Speak the autocompleted text, but only if it is different
         # address so that we're not too "chatty." See bug #533042.
-        if parentRole == pyatspi.ROLE_AUTOCOMPLETE:
+        if AXUtilities.is_autocomplete(parent):
             if len(event.any_data) == 1:
                 default.Script.onTextInserted(self, event)
                 return
@@ -319,13 +290,7 @@ class Script(Gecko.Script):
 
             # Mozilla cannot seem to get their ":system" suffix right
             # to save their lives, so we'll add yet another sad hack.
-            try:
-                text = event.source.queryText()
-            except:
-                hasSelection = False
-            else:
-                hasSelection = text.getNSelections() > 0
-            if hasSelection or isSystemEvent:
+            if isSystemEvent or AXText.has_selected_text(event.source):
                 voice = self.speechGenerator.voice(obj=event.source, string=event.any_data)
                 self.speakMessage(event.any_data, voice=voice)
                 self._lastAutoComplete = event.any_data
@@ -342,9 +307,9 @@ class Script(Gecko.Script):
             return
 
         if self.utilities.isEditableMessage(obj) and self.spellcheck.isActive():
-            text = obj.queryText()
-            selStart, selEnd = text.getSelection(0)
-            self.spellcheck.setDocumentPosition(obj, selStart)
+            selStart = AXText.get_selection_start_offset(obj)
+            if selStart >= 0:
+                self.spellcheck.setDocumentPosition(obj, selStart)
             return
 
         super().onTextSelectionChanged(event)
@@ -352,14 +317,10 @@ class Script(Gecko.Script):
     def onNameChanged(self, event):
         """Callback for object:property-change:accessible-name events."""
 
-        if event.source.name == self.spellcheck.getMisspelledWord():
+        if AXObject.get_name(event.source) == self.spellcheck.getMisspelledWord():
             self.spellcheck.presentErrorDetails()
             return
 
-        if not self.utilities.lastInputEventWasDelete() \
-           or not self.utilities.isDocument(event.source):
-            return
-
         super().onNameChanged(event)
 
     def _presentMessage(self, documentFrame):
@@ -370,35 +331,25 @@ class Script(Gecko.Script):
         self.utilities.setCaretPosition(obj, offset)
         self.updateBraille(obj)
 
-        if _settingsManager.getSetting('pageSummaryOnLoad'):
-            msg = "THUNDERBIRD: Getting page summary for obj %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            summary = self.utilities.getPageSummary(obj)
+        if settings_manager.getManager().getSetting('pageSummaryOnLoad'):
+            tokens = ["THUNDERBIRD: Getting page summary for", documentFrame]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            summary = AXDocument.get_document_summary(documentFrame)
             if summary:
                 self.presentMessage(summary)
 
-        if not _settingsManager.getSetting('sayAllOnLoad'):
+        if not settings_manager.getManager().getSetting('sayAllOnLoad'):
             msg = "THUNDERBIRD: SayAllOnLoad is False. Presenting line."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             contents = self.utilities.getLineContentsAtOffset(obj, offset)
             self.speakContents(contents)
             return
 
-        if _settingsManager.getSetting('enableSpeech'):
+        if settings_manager.getManager().getSetting('enableSpeech'):
             msg = "THUNDERBIRD: SayAllOnLoad is True and speech is enabled"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             self.sayAll(None)
 
-    def toggleFlatReviewMode(self, inputEvent=None):
-        """Toggles between flat review mode and focus tracking mode."""
-
-        # If we're leaving flat review dump the cache. See bug 568658.
-        #
-        if self.flatReviewContext:
-            pyatspi.clearCache()
-
-        return default.Script.toggleFlatReviewMode(self, inputEvent)
-
     def onWindowActivated(self, event):
         """Callback for window:activate accessibility events."""
 
@@ -408,8 +359,9 @@ class Script(Gecko.Script):
             return
 
         self.spellcheck.presentErrorDetails()
-        orca.setLocusOfFocus(None, self.spellcheck.getChangeToEntry(), False)
-        self.updateBraille(orca_state.locusOfFocus)
+        entry = self.spellcheck.getChangeToEntry()
+        focus_manager.getManager().set_locus_of_focus(None, entry, False)
+        self.updateBraille(entry)
 
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
