@@ -27,19 +27,19 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc." \
                 "Copyright (c) 2013 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
 
-import orca.orca as orca
+import orca.debug as debug
+import orca.focus_manager as focus_manager
 import orca.scripts.toolkits.gtk as gtk
 import orca.scripts.toolkits.WebKitGtk as WebKitGtk
-import orca.settings as settings
 import orca.settings_manager as settings_manager
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 from .braille_generator import BrailleGenerator
 from .speech_generator import SpeechGenerator
 from .script_utilities import Utilities
 
-_settingsManager = settings_manager.getManager()
 
 ########################################################################
 #                                                                      #
@@ -56,8 +56,8 @@ class Script(WebKitGtk.Script, gtk.Script):
         - app: the application to create a script for.
         """
 
-        if _settingsManager.getSetting('sayAllOnLoad') is None:
-            _settingsManager.setSetting('sayAllOnLoad', False)
+        if settings_manager.getManager().getSetting('sayAllOnLoad') is None:
+            settings_manager.getManager().setSetting('sayAllOnLoad', False)
 
         super().__init__(app)
         self.presentIfInactive = False
@@ -81,8 +81,11 @@ class Script(WebKitGtk.Script, gtk.Script):
         to say it shouldn't.
         """
 
+        if event.type.startswith("focus:") and AXUtilities.is_menu(event.source):
+            return True
+
         window = self.utilities.topLevelObject(event.source)
-        if window and not window.getState().contains(pyatspi.STATE_ACTIVE):
+        if not AXUtilities.is_active(window):
             return False
 
         return True
@@ -109,15 +112,38 @@ class Script(WebKitGtk.Script, gtk.Script):
         """Callback for object:active-descendant-changed accessibility events."""
 
         if not event.any_data:
+            msg = "EVOLUTION: Ignoring event. No any_data."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         if self.utilities.isComposeAutocomplete(event.source):
-            if event.any_data.getState().contains(pyatspi.STATE_SELECTED):
-                orca.setLocusOfFocus(event, event.any_data)
+            if AXUtilities.is_selected(event.any_data):
+                msg = "EVOLUTION: Source is compose autocomplete with selected child."
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+                focus_manager.getManager().set_locus_of_focus(event, event.any_data)
             else:
-                orca.setLocusOfFocus(event, event.source)
+                msg = "EVOLUTION: Source is compose autocomplete without selected child."
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+                focus_manager.getManager().set_locus_of_focus(event, event.source)
             return
 
+        focus = focus_manager.getManager().get_locus_of_focus()
+        if AXUtilities.is_table_cell(focus):
+            table = AXObject.find_ancestor(focus, AXUtilities.is_tree_or_tree_table)
+            if table is not None and table != event.source:
+                msg = "EVOLUTION: Event is from a different tree or tree table."
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+                return
+
+        child = AXObject.get_active_descendant_checked(event.source, event.any_data)
+        if child is not None and child != event.any_data:
+            tokens = ["EVOLUTION: Bogus any_data suspected. Setting focus to", child]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            focus_manager.getManager().set_locus_of_focus(event, child)
+            return
+
+        msg = "EVOLUTION: Passing event to super class for processing."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         super().onActiveDescendantChanged(event)
 
     def onBusyChanged(self, event):
@@ -132,9 +158,9 @@ class Script(WebKitGtk.Script, gtk.Script):
 
         # This is some mystery child of the 'Messages' panel which fails to show
         # up in the hierarchy or emit object:state-changed:focused events.
-        if event.source.getRole() == pyatspi.ROLE_LAYERED_PANE:
+        if AXUtilities.is_layered_pane(event.source):
             obj = self.utilities.realActiveDescendant(event.source)
-            orca.setLocusOfFocus(event, obj)
+            focus_manager.getManager().set_locus_of_focus(event, obj)
             return
 
         gtk.Script.onFocus(self, event)
@@ -150,9 +176,8 @@ class Script(WebKitGtk.Script, gtk.Script):
     def onSelectionChanged(self, event):
         """Callback for object:selection-changed accessibility events."""
 
-        obj = event.source
-        if obj.getRole() == pyatspi.ROLE_COMBO_BOX \
-           and not obj.getState().contains(pyatspi.STATE_FOCUSED):
+        if AXUtilities.is_combo_box(event.source) \
+           and not AXUtilities.is_focused(event.source):
             return
 
         gtk.Script.onSelectionChanged(self, event)
