@@ -25,44 +25,53 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2015 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
-
 import orca.scripts.toolkits.gtk as gtk
 import orca.scripts.toolkits.WebKitGtk as WebKitGtk
+from orca.ax_object import AXObject
+from orca.ax_table import AXTable
+from orca.ax_utilities import AXUtilities
 
-class Utilities(WebKitGtk.Utilities):
+
+class Utilities(WebKitGtk.Utilities, gtk.Utilities):
 
     def __init__(self, script):
         super().__init__(script)
 
     def isComposeMessageBody(self, obj):
-        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
+        if not AXUtilities.is_editable(obj):
             return False
 
         return self.isEmbeddedDocument(obj)
 
     def isReceivedMessage(self, obj):
-        if obj.getState().contains(pyatspi.STATE_EDITABLE):
+        if AXUtilities.is_editable(obj):
             return False
 
         return self.isEmbeddedDocument(obj)
 
     def isReceivedMessageHeader(self, obj):
-        if not (obj and obj.getRole() == pyatspi.ROLE_TABLE):
+        if not AXUtilities.is_table(obj):
             return False
 
-        return self.isReceivedMessage(obj.parent)
+        return self.isReceivedMessage(AXObject.get_parent(obj))
 
     def isReceivedMessageContent(self, obj):
-        if not (obj and obj.getRole() == pyatspi.ROLE_SECTION):
+        if not AXUtilities.is_section(obj):
             return False
 
-        return self.isReceivedMessage(obj.parent)
+        return self.isReceivedMessage(AXObject.get_parent(obj))
+
+    def isComposeAutocomplete(self, obj):
+        if not AXUtilities.is_table(obj):
+            return False
+
+        if not AXUtilities.manages_descendants(obj):
+            return False
+
+        return AXUtilities.is_window(self.topLevelObject(obj))
 
     def findMessageBodyChild(self, root):
-        roles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        isDocument = lambda x: x and x.getRole() in roles
-        candidate = pyatspi.findDescendant(root, isDocument)
+        candidate = AXObject.find_descendant(root, self.isDocument)
         if self.isEmbeddedDocument(candidate):
             return self.findMessageBodyChild(candidate)
 
@@ -72,8 +81,11 @@ class Utilities(WebKitGtk.Utilities):
         if not self.isMessageListToggleCell(obj):
             return False
 
-        header = self.columnHeaderForCell(obj)
-        return header and header.name != obj.name
+        headers = AXTable.get_column_headers(obj)
+        if not headers:
+            return False
+
+        return headers[0] and AXObject.get_name(headers[0]) != AXObject.get_name(obj)
 
     def isMessageListToggleCell(self, obj):
         if self.isWebKitGtk(obj):
@@ -82,7 +94,7 @@ class Utilities(WebKitGtk.Utilities):
         if not gtk.Utilities.hasMeaningfulToggleAction(self, obj):
             return False
 
-        if not obj.name:
+        if not AXObject.get_name(obj):
             return False
 
         return True
@@ -93,9 +105,8 @@ class Utilities(WebKitGtk.Utilities):
 
         # This is some mystery child of the 'Messages' panel which fails to show
         # up in the hierarchy or emit object:state-changed:focused events.
-        if obj.getRole() == pyatspi.ROLE_LAYERED_PANE:
-            isTreeTable = lambda x: x and x.getRole() == pyatspi.ROLE_TREE_TABLE
-            return pyatspi.utils.findDescendant(obj, isTreeTable) or obj
+        if AXUtilities.is_layered_pane(obj):
+            return AXObject.find_descendant(obj, AXUtilities.is_tree_table) or obj
 
         return gtk.Utilities.realActiveDescendant(self, obj)
 
@@ -113,8 +124,7 @@ class Utilities(WebKitGtk.Utilities):
         if not self.isEmbeddedDocument(obj):
             return False
 
-        isSplitPane = lambda x: x and x.getRole() == pyatspi.ROLE_SPLIT_PANE
-        if pyatspi.utils.findAncestor(obj, isSplitPane):
+        if AXObject.find_ancestor(obj, AXUtilities.is_split_pane) is not None:
             return False
 
         return True
