@@ -25,10 +25,13 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Joanmarie Diggs."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 
 import orca.chat as chat
 import orca.scripts.default as default
+from orca.ax_utilities import AXUtilities
 
 ########################################################################
 #                                                                      #
@@ -43,12 +46,12 @@ class Script(default.Script):
 
         # So we can take an educated guess at identifying the buddy list.
         #
-        self._buddyListAncestries = [[pyatspi.ROLE_TABLE,
-                                      pyatspi.ROLE_SCROLL_PANE,
-                                      pyatspi.ROLE_FILLER,
-                                      pyatspi.ROLE_SPLIT_PANE,
-                                      pyatspi.ROLE_FILLER,
-                                      pyatspi.ROLE_FRAME]]
+        self._buddyListAncestries = [[Atspi.Role.TABLE,
+                                      Atspi.Role.SCROLL_PANE,
+                                      Atspi.Role.FILLER,
+                                      Atspi.Role.SPLIT_PANE,
+                                      Atspi.Role.FILLER,
+                                      Atspi.Role.FRAME]]
 
         default.Script.__init__(self, app)
 
@@ -97,8 +100,5 @@ class Script(default.Script):
         # Hack to "tickle" the accessible hierarchy. Otherwise, the
         # events we need to present text added to the chatroom are
         # missing.
-        #
-        allPageTabs = self.utilities.descendantsWithRole(
-            event.source, pyatspi.ROLE_PAGE_TAB)
-
+        AXUtilities.find_all_page_tabs(event.source)
         default.Script.onWindowActivated(self, event)
