@@ -24,21 +24,16 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2016 Igalia, S.L."
 __license__   = "LGPL"
 
-import gi
-gi.require_version("Atspi", "2.0")
-from gi.repository import Atspi
-
 import re
 
 from orca import debug
+from orca import focus_manager
 from orca import keybindings
-from orca import orca_state
 from orca import script_utilities
 from orca import settings_manager
+from orca.ax_text import AXText
 from orca.ax_utilities import AXUtilities
 
-_settingsManager = settings_manager.getManager()
-
 
 class Utilities(script_utilities.Utilities):
 
@@ -68,32 +63,23 @@ class Utilities(script_utilities.Utilities):
         if self.isClipboardTextChangedEvent(event):
             return event.any_data
 
-        try:
-            text = event.source.queryText()
-        except Exception:
-            tokens = ["ERROR: Exception querying text for", event.source]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return event.any_data
-
         start, end = event.detail1, event.detail1 + len(event.any_data)
-        boundary = Atspi.TextBoundaryType.LINE_START
+        firstLine = AXText.get_line_at_offset(event.source, start)
+        tokens = ["TERMINAL: First line of insertion:", firstLine]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        firstLine = text.getTextAtOffset(start, boundary)
-        msg = "TERMINAL: First line of insertion: '%s' (%i, %i)" % firstLine
-        debug.println(debug.LEVEL_INFO, msg, True)
-
-        lastLine = text.getTextAtOffset(end - 1, boundary)
-        msg = "TERMINAL: Last line of insertion: '%s' (%i, %i)" % lastLine
-        debug.println(debug.LEVEL_INFO, msg, True)
+        lastLine = AXText.get_line_at_offset(event.source, end - 1)
+        tokens = ["TERMINAL: Last line of insertion:", lastLine]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if firstLine == lastLine:
             msg = "TERMINAL: Not adjusting single-line insertion."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return event.any_data
 
-        currentLine = text.getTextAtOffset(text.caretOffset, boundary)
-        msg = "TERMINAL: Current line: '%s' (%i, %i)" % currentLine
-        debug.println(debug.LEVEL_INFO, msg, True)
+        currentLine = AXText.get_line_at_offset(event.source, None)
+        tokens = ["TERMINAL: Current line:", currentLine]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if firstLine != ("", 0, 0):
             start = firstLine[1]
@@ -106,7 +92,7 @@ class Utilities(script_utilities.Utilities):
             if lastLine[0].endswith("\n"):
                 end -= 1
 
-        adjusted = text.getText(start, end)
+        adjusted = AXText.get_substring(event.source, start, end)
         if adjusted:
             tokens = ["TERMINAL: Adjusted insertion: '", adjusted, "'"]
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
@@ -118,14 +104,7 @@ class Utilities(script_utilities.Utilities):
         return adjusted
 
     def insertionEndsAtCaret(self, event):
-        try:
-            text = event.source.queryText()
-        except Exception:
-            tokens = ["ERROR: Exception querying text for", event.source]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return False
-
-        return text.caretOffset == event.detail1 + event.detail2
+        return AXText.get_caret_offset(event.source) == event.detail1 + event.detail2
 
     def isEditableTextArea(self, obj):
         if AXUtilities.is_terminal(obj):
@@ -180,7 +159,7 @@ class Utilities(script_utilities.Utilities):
         return False
 
     def treatEventAsCommand(self, event):
-        if event.source != orca_state.locusOfFocus:
+        if event.source != focus_manager.getManager().get_locus_of_focus():
             return False
 
         if event.type.startswith("object:text-changed:insert") and event.any_data.strip():
@@ -198,7 +177,7 @@ class Utilities(script_utilities.Utilities):
                 return True
             if self.lastInputEventWasPrintableKey():
                 return len(event.any_data) > 1
-            if self.insertionEndsAtCaret(event):
+            if AXText.get_caret_offset(event.source) == event.detail1 + event.detail2:
                 return True
 
         return False
@@ -219,7 +198,7 @@ class Utilities(script_utilities.Utilities):
         return False
 
     def willEchoCharacter(self, event):
-        if not _settingsManager.getSetting("enableEchoByCharacter"):
+        if not settings_manager.getManager().getSetting("enableEchoByCharacter"):
             return False
 
         if len(event.event_string) != 1 \
