@@ -28,7 +28,9 @@ __copyright__ = "Copyright (c) 2018-2019 Igalia, S.L."
 __license__   = "LGPL"
 
 from orca import debug
-from orca import orca
+from orca import focus_manager
+from orca.ax_component import AXComponent
+from orca.ax_document import AXDocument
 from orca.ax_object import AXObject
 from orca.ax_utilities import AXUtilities
 from orca.scripts import default
@@ -60,14 +62,6 @@ class Script(web.Script):
 
         return Utilities(self)
 
-    def isActivatableEvent(self, event):
-        """Returns True if this event should activate this script."""
-
-        if event.type == "window:activate":
-            return self.utilities.canBeActiveWindow(event.source)
-
-        return super().isActivatableEvent(event)
-
     def locusOfFocusChanged(self, event, oldFocus, newFocus):
         """Handles changes of focus of interest to the script."""
 
@@ -85,7 +79,7 @@ class Script(web.Script):
             return
 
         if event.detail1 and AXUtilities.is_frame(event.source) \
-           and not self.utilities.canBeActiveWindow(event.source):
+           and not focus_manager.getManager().can_be_active_window(event.source):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
@@ -105,12 +99,12 @@ class Script(web.Script):
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
-        if self.utilities.hasNoSize(event.source):
+        if AXComponent.has_no_size(event.source):
             msg = "CHROMIUM: Ignoring event from page with no size."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
-        if not self.utilities.documentFrameURI(event.source):
+        if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -195,7 +189,7 @@ class Script(web.Script):
     def onDocumentLoadComplete(self, event):
         """Callback for document:load-complete accessibility events."""
 
-        if not self.utilities.documentFrameURI(event.source):
+        if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -210,7 +204,7 @@ class Script(web.Script):
     def onDocumentLoadStopped(self, event):
         """Callback for document:load-stopped accessibility events."""
 
-        if not self.utilities.documentFrameURI(event.source):
+        if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -225,7 +219,7 @@ class Script(web.Script):
     def onDocumentReload(self, event):
         """Callback for document:reload accessibility events."""
 
-        if not self.utilities.documentFrameURI(event.source):
+        if not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from page with no URI."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -264,7 +258,7 @@ class Script(web.Script):
         """Callback for object:state-changed:focused accessibility events."""
 
         if self.utilities.isDocument(event.source) \
-           and not self.utilities.documentFrameURI(event.source):
+           and not AXDocument.get_uri(event.source):
             msg = "CHROMIUM: Ignoring event from document with no URI."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -329,14 +323,6 @@ class Script(web.Script):
     def onShowingChanged(self, event):
         """Callback for object:state-changed:showing accessibility events."""
 
-        if event.detail1 and self.utilities.isMenuWithNoSelectedChild(event.source):
-            topLevel = self.utilities.topLevelObject(event.source)
-            if self.utilities.canBeActiveWindow(topLevel):
-                orca.setActiveWindow(topLevel)
-                self.presentObject(event.source)
-                orca.setLocusOfFocus(event, event.source, False)
-            return
-
         if super().onShowingChanged(event):
             return
 
@@ -397,7 +383,7 @@ class Script(web.Script):
     def onWindowActivated(self, event):
         """Callback for window:activate accessibility events."""
 
-        if not self.utilities.canBeActiveWindow(event.source):
+        if not focus_manager.getManager().can_be_active_window(event.source):
             return
 
         # If this is a frame for a popup menu, we don't want to treat
@@ -405,7 +391,7 @@ class Script(web.Script):
         # far as the end-user experience is concerned.
         menu = self.utilities.popupMenuForFrame(event.source)
         if menu:
-            orca.setActiveWindow(event.source)
+            focus_manager.getManager().set_active_window(event.source)
 
             activeItem = None
             selected = self.utilities.selectedChildren(menu)
@@ -416,16 +402,16 @@ class Script(web.Script):
                 # If this is the popup menu for the locusOfFocus, we don't want to
                 # present the popup menu as part of the new ancestry of activeItem.
                 if self.utilities.isPopupMenuForCurrentItem(menu):
-                    orca.setLocusOfFocus(event, menu, False)
+                    focus_manager.getManager().set_locus_of_focus(event, menu, False)
 
                 tokens = ["CHROMIUM: Setting locusOfFocus to active item", activeItem]
                 debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                orca.setLocusOfFocus(event, activeItem)
+                focus_manager.getManager().set_locus_of_focus(event, activeItem)
                 return
 
             tokens = ["CHROMIUM: Setting locusOfFocus to popup menu", menu]
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            orca.setLocusOfFocus(event, menu)
+            focus_manager.getManager().set_locus_of_focus(event, menu)
 
         if super().onWindowActivated(event):
             return
