@@ -27,30 +27,30 @@ __copyright__ = "Copyright (C) 2010-2011 The Orca Team" \
                 "Copyright (C) 2011-2012 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
-import pyatspi.utils as utils
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 
 import orca.scripts.default as default
 import orca.cmdnames as cmdnames
 import orca.debug as debug
-import orca.eventsynthesizer as eventsynthesizer
+import orca.focus_manager as focus_manager
 import orca.guilabels as guilabels
 import orca.input_event as input_event
 import orca.messages as messages
-import orca.orca as orca
 import orca.settings as settings
 import orca.settings_manager as settings_manager
-import orca.speechserver as speechserver
-import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.structural_navigation as structural_navigation
 
+from orca.ax_component import AXComponent
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
+
 from .braille_generator import BrailleGenerator
 from .speech_generator import SpeechGenerator
 from .script_utilities import Utilities
 
-_settingsManager = settings_manager.getManager()
-
 ########################################################################
 #                                                                      #
 # The WebKitGtk script class.                                          #
@@ -71,16 +71,15 @@ class Script(default.Script):
         self._lastCaretContext = None, -1
         self.sayAllOnLoadCheckButton = None
 
-        if _settingsManager.getSetting('sayAllOnLoad') is None:
-            _settingsManager.setSetting('sayAllOnLoad', True)
+        if settings_manager.getManager().getSetting('sayAllOnLoad') is None:
+            settings_manager.getManager().setSetting('sayAllOnLoad', True)
 
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
         called by the key and braille bindings."""
 
         default.Script.setupInputEventHandlers(self)
-        self.inputEventHandlers.update(
-            self.structuralNavigation.inputEventHandlers)
+        self.inputEventHandlers.update(self.structuralNavigation.get_handlers(True))
 
         self.inputEventHandlers["sayAllHandler"] = \
             input_event.InputEventHandler(
@@ -102,7 +101,9 @@ class Script(default.Script):
     def getToolkitKeyBindings(self):
         """Returns the toolkit-specific keybindings for this script."""
 
-        return self.structuralNavigation.keyBindings
+        layout = settings_manager.getManager().getSetting('keyboardLayout')
+        isDesktop = layout == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP
+        return self.structuralNavigation.get_bindings(refresh=True, is_desktop=isDesktop)
 
     def getAppPreferencesGUI(self):
         """Return a GtkGrid containing the application unique configuration
@@ -117,7 +118,7 @@ class Script(default.Script):
         self.sayAllOnLoadCheckButton = \
             Gtk.CheckButton.new_with_mnemonic(label)
         self.sayAllOnLoadCheckButton.set_active(
-            _settingsManager.getSetting('sayAllOnLoad'))
+            settings_manager.getManager().getSetting('sayAllOnLoad'))
         grid.attach(self.sayAllOnLoadCheckButton, 0, 0, 1, 1)
 
         grid.show_all()
@@ -153,6 +154,7 @@ class Script(default.Script):
                 structural_navigation.StructuralNavigation.ENTRY,
                 structural_navigation.StructuralNavigation.FORM_FIELD,
                 structural_navigation.StructuralNavigation.HEADING,
+                structural_navigation.StructuralNavigation.IFRAME,
                 structural_navigation.StructuralNavigation.IMAGE,
                 structural_navigation.StructuralNavigation.LANDMARK,
                 structural_navigation.StructuralNavigation.LINK,
@@ -163,12 +165,11 @@ class Script(default.Script):
                 structural_navigation.StructuralNavigation.RADIO_BUTTON,
                 structural_navigation.StructuralNavigation.SEPARATOR,
                 structural_navigation.StructuralNavigation.TABLE,
-                structural_navigation.StructuralNavigation.TABLE_CELL,
                 structural_navigation.StructuralNavigation.UNVISITED_LINK,
                 structural_navigation.StructuralNavigation.VISITED_LINK]
 
     def getUtilities(self):
-        """Returns the utilites for this script."""
+        """Returns the utilities for this script."""
 
         return Utilities(self)
 
@@ -186,10 +187,11 @@ class Script(default.Script):
         if lastKey in ['Tab', 'ISO_Left_Tab']:
             return
 
+        focus = focus_manager.getManager().get_locus_of_focus()
         if lastKey == 'Down' \
-           and orca_state.locusOfFocus == event.source.parent \
-           and event.source.getIndexInParent() == 0 \
-           and orca_state.locusOfFocus.getRole() == pyatspi.ROLE_LINK:
+           and AXObject.get_index_in_parent(event.source) == 0 \
+           and focus == AXObject.get_parent(event.source) \
+           and AXUtilities.is_link(focus):
             self.updateBraille(event.source)
             return
 
@@ -217,8 +219,8 @@ class Script(default.Script):
         self.utilities.setCaretContext(obj, offset)
 
         self.updateBraille(obj)
-        if _settingsManager.getSetting('sayAllOnLoad') \
-           and _settingsManager.getSetting('enableSpeech'):
+        if settings_manager.getManager().getSetting('sayAllOnLoad') \
+           and settings_manager.getManager().getSetting('enableSpeech'):
             self.sayAll(None)
 
     def onDocumentLoadStopped(self, event):
@@ -242,14 +244,14 @@ class Script(default.Script):
             return
 
         obj = event.source
-        role = obj.getRole()
-        textRoles = [pyatspi.ROLE_HEADING,
-                     pyatspi.ROLE_PANEL,
-                     pyatspi.ROLE_PARAGRAPH,
-                     pyatspi.ROLE_SECTION,
-                     pyatspi.ROLE_TABLE_CELL]
+        role = AXObject.get_role(obj)
+        textRoles = [Atspi.Role.HEADING,
+                     Atspi.Role.PANEL,
+                     Atspi.Role.PARAGRAPH,
+                     Atspi.Role.SECTION,
+                     Atspi.Role.TABLE_CELL]
         if role in textRoles \
-           or (role == pyatspi.ROLE_LIST_ITEM and obj.childCount):
+           or (role == Atspi.Role.LIST_ITEM and AXObject.get_child_count(obj)):
             return
 
         super().onFocusedChanged(event)
@@ -257,19 +259,15 @@ class Script(default.Script):
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
-        obj = event.source
-        try:
-            role = obj.getRole()
-            name = obj.name
-        except:
-            return
-
-        if not self.utilities.treatAsBrowser(obj):
+        if not self.utilities.treatAsBrowser(event.source):
             return
 
         if event.detail1:
             self.presentMessage(messages.PAGE_LOADING_START)
-        elif name:
+            return
+
+        name = AXObject.get_name(event.source)
+        if name:
             self.presentMessage(messages.PAGE_LOADING_END_NAMED % name)
         else:
             self.presentMessage(messages.PAGE_LOADING_END)
@@ -281,11 +279,11 @@ class Script(default.Script):
         - obj: an Accessible object that implements the AccessibleText interface
         """
 
-        if obj.getRole() == pyatspi.ROLE_ENTRY:
+        if AXUtilities.is_entry(obj):
             default.Script.sayCharacter(self, obj)
             return
 
-        boundary = pyatspi.TEXT_BOUNDARY_CHAR
+        boundary = Atspi.TextBoundaryType.CHAR
         objects = self.utilities.getObjectsFromEOCs(obj, boundary=boundary)
         for (obj, start, end, string) in objects:
             if string:
@@ -302,11 +300,11 @@ class Script(default.Script):
         - obj: an Accessible object that implements the AccessibleText interface
         """
 
-        if obj.getRole() == pyatspi.ROLE_ENTRY:
+        if AXUtilities.is_entry(obj):
             default.Script.sayWord(self, obj)
             return
 
-        boundary = pyatspi.TEXT_BOUNDARY_WORD_START
+        boundary = Atspi.TextBoundaryType.WORD_START
         objects = self.utilities.getObjectsFromEOCs(obj, boundary=boundary)
         for (obj, start, end, string) in objects:
             self.sayPhrase(obj, start, end)
@@ -320,22 +318,21 @@ class Script(default.Script):
         - obj: an Accessible object that implements the AccessibleText interface
         """
 
-        if obj.getRole() == pyatspi.ROLE_ENTRY:
+        if AXUtilities.is_entry(obj):
             default.Script.sayLine(self, obj)
             return
 
-        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
+        boundary = Atspi.TextBoundaryType.LINE_START
         objects = self.utilities.getObjectsFromEOCs(obj, boundary=boundary)
         for (obj, start, end, string) in objects:
             self.sayPhrase(obj, start, end)
 
             # TODO: Move these next items into the speech generator.
-            if obj.getRole() == pyatspi.ROLE_PANEL \
-               and obj.getIndexInParent() == 0:
-                obj = obj.parent
+            if AXUtilities.is_panel(obj) and AXObject.get_index_in_parent(obj) == 0:
+                obj = AXObject.get_parent(obj)
 
-            rolesToSpeak = [pyatspi.ROLE_HEADING, pyatspi.ROLE_LINK]
-            if obj.getRole() in rolesToSpeak:
+            rolesToSpeak = [Atspi.Role.HEADING, Atspi.Role.LINK]
+            if AXObject.get_role(obj) in rolesToSpeak:
                 speech.speak(self.speechGenerator.getRoleName(obj))
 
         self.pointOfReference["lastTextUnitSpoken"] = "line"
@@ -349,7 +346,7 @@ class Script(default.Script):
         - endOffset: the end text offset.
         """
 
-        if obj.getRole() == pyatspi.ROLE_ENTRY:
+        if AXUtilities.is_entry(obj):
             default.Script.sayPhrase(self, obj, startOffset, endOffset)
             return
 
@@ -357,7 +354,7 @@ class Script(default.Script):
         if len(phrase) and phrase != "\n":
             voice = self.speechGenerator.voice(obj=obj, string=phrase)
             phrase = self.utilities.adjustForRepeats(phrase)
-            links = [x for x in obj if x.getRole() == pyatspi.ROLE_LINK]
+            links = [x for x in AXObject.iter_children(obj, AXUtilities.is_link)]
             if links:
                 phrase = self.utilities.adjustForLinks(obj, phrase, startOffset)
             speech.speak(phrase, voice)
@@ -378,58 +375,25 @@ class Script(default.Script):
         Returns True if we shouldn't bother processing this object event.
         """
 
-        if event.type.startswith('object:state-changed:focused') \
-           and event.detail1:
-            if event.source.getRole() == pyatspi.ROLE_LINK:
+        if event.type.startswith('object:state-changed:focused') and event.detail1 \
+           and AXUtilities.is_link(event.source):
                 return False
 
         return default.Script.skipObjectEvent(self, event)
 
-    def useStructuralNavigationModel(self):
-        """Returns True if we should do our own structural navigation.
-        This should return False if we're in a form field, or not in
-        document content.
-        """
-
-        doNotHandleRoles = [pyatspi.ROLE_ENTRY,
-                            pyatspi.ROLE_TEXT,
-                            pyatspi.ROLE_PASSWORD_TEXT,
-                            pyatspi.ROLE_LIST,
-                            pyatspi.ROLE_LIST_ITEM,
-                            pyatspi.ROLE_MENU_ITEM]
-
-        if not self.structuralNavigation.enabled:
-            return False
-
-        if not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
-            return False
-
-        states = orca_state.locusOfFocus.getState()
-        if states.contains(pyatspi.STATE_EDITABLE):
-            return False
-
-        role = orca_state.locusOfFocus.getRole()
-        if role in doNotHandleRoles:
-            if role == pyatspi.ROLE_LIST_ITEM:
-                return not states.contains(pyatspi.STATE_SELECTABLE)
-
-            if states.contains(pyatspi.STATE_FOCUSED):
-                return False
-
-        return True
-
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
         """
 
-        if self.flatReviewContext \
+        focus = focus_manager.getManager().get_locus_of_focus()
+        if self.flatReviewPresenter.is_active() \
            or not self.isBrailleBeginningShowing() \
-           or not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
+           or not self.utilities.isWebKitGtk(focus):
             return default.Script.panBrailleLeft(self, inputEvent, panAmount)
 
-        obj = self.utilities.findPreviousObject(orca_state.locusOfFocus)
-        orca.setLocusOfFocus(None, obj, notifyScript=False)
+        obj = self.utilities.findPreviousObject(focus)
+        focus_manager.getManager().set_locus_of_focus(None, obj, notify_script=False)
         self.updateBraille(obj)
 
         # Hack: When panning to the left in a document, we want to start at
@@ -446,13 +410,14 @@ class Script(default.Script):
         entire document.
         """
 
-        if self.flatReviewContext \
+        focus = focus_manager.getManager().get_locus_of_focus()
+        if self.flatReviewPresenter.is_active() \
            or not self.isBrailleEndShowing() \
-           or not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
+           or not self.utilities.isWebKitGtk(focus):
             return default.Script.panBrailleRight(self, inputEvent, panAmount)
 
-        obj = self.utilities.findNextObject(orca_state.locusOfFocus)
-        orca.setLocusOfFocus(None, obj, notifyScript=False)
+        obj = self.utilities.findNextObject(focus)
+        focus_manager.getManager().set_locus_of_focus(None, obj, notify_script=False)
         self.updateBraille(obj)
 
         # Hack: When panning to the right in a document, we want to start at
@@ -470,158 +435,13 @@ class Script(default.Script):
         been started on an object without text (such as an image).
         """
 
-        obj = obj or orca_state.locusOfFocus
+        obj = obj or focus_manager.getManager().get_locus_of_focus()
         if not self.utilities.isWebKitGtk(obj):
             return default.Script.sayAll(self, inputEvent, obj, offset)
 
-        speech.sayAll(self.textLines(obj, offset),
-                      self.__sayAllProgressCallback)
-
+        speech.sayAll(self.textLines(obj, offset), self.__sayAllProgressCallback)
         return True
 
-    def getTextSegments(self, obj, boundary, offset=0):
-        segments = []
-        text = obj.queryText()
-        length = text.characterCount
-        string, start, end = text.getTextAtOffset(offset, boundary)
-        while string and offset < length:
-            string = self.utilities.adjustForRepeats(string)
-            voice = self.speechGenerator.getVoiceForString(obj, string)
-            string = self.utilities.adjustForLinks(obj, string, start)
-            # Incrementing the offset should cause us to eventually reach
-            # the end of the text as indicated by a 0-length string and
-            # start and end offsets of 0. Sometimes WebKitGtk returns the
-            # final text segment instead.
-            if segments and [string, start, end, voice] == segments[-1]:
-                break
-
-            segments.append([string, start, end, voice])
-            offset = end + 1
-            string, start, end = text.getTextAtOffset(offset, boundary)
-        return segments
-
-    def textLines(self, obj, offset=None):
-        """Creates a generator that can be used to iterate over each line
-        of a text object, starting at the caret offset.
-
-        Arguments:
-        - obj: an Accessible that has a text specialization
-
-        Returns an iterator that produces elements of the form:
-        [SayAllContext, acss], where SayAllContext has the text to be
-        spoken and acss is an ACSS instance for speaking the text.
-        """
-
-        self._sayAllIsInterrupted = False
-        self._inSayAll = False
-        if not obj:
-            return
-
-        if obj.getRole() == pyatspi.ROLE_LINK:
-            obj = obj.parent
-
-        document = self.utilities.getDocumentForObject(obj)
-        if not document or document.getState().contains(pyatspi.STATE_BUSY):
-            return
-
-        allTextObjs = utils.findAllDescendants(
-            document, lambda x: x and 'Text' in utils.listInterfaces(x))
-        allTextObjs = allTextObjs[allTextObjs.index(obj):len(allTextObjs)]
-        textObjs = [x for x in allTextObjs if x.parent not in allTextObjs]
-        if not textObjs:
-            return
-
-        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-        sayAllStyle = _settingsManager.getSetting('sayAllStyle')
-        if sayAllStyle == settings.SAYALL_STYLE_SENTENCE:
-            boundary = pyatspi.TEXT_BOUNDARY_SENTENCE_START
-
-        voices = _settingsManager.getSetting('voices')
-        systemVoice = voices.get(settings.SYSTEM_VOICE)
-
-        self._inSayAll = True
-        offset = textObjs[0].queryText().caretOffset
-        for textObj in textObjs:
-            textSegments = self.getTextSegments(textObj, boundary, offset)
-            roleName = self.speechGenerator.getRoleName(textObj)
-            if roleName:
-                textSegments.append([roleName, 0, -1, systemVoice])
-
-            for (string, start, end, voice) in textSegments:
-                context = speechserver.SayAllContext(textObj, string, start, end)
-                self._sayAllContexts.append(context)
-                eventsynthesizer.scrollIntoView(obj, start, end)
-                yield [context, voice]
-
-            offset = 0
-
-        self._inSayAll = False
-        self._sayAllContexts = []
-
-    def __sayAllProgressCallback(self, context, progressType):
-        if progressType == speechserver.SayAllContext.PROGRESS:
-            orca.emitRegionChanged(
-                context.obj, context.currentOffset, context.currentEndOffset, orca.SAY_ALL)
-            return
-
-        obj = context.obj
-        orca.setLocusOfFocus(None, obj, notifyScript=False)
-
-        offset = context.currentOffset
-        text = obj.queryText()
-
-        if progressType == speechserver.SayAllContext.INTERRUPTED:
-            self._sayAllIsInterrupted = True
-            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
-                lastKey = orca_state.lastInputEvent.event_string
-                if lastKey == "Down" and self._fastForwardSayAll(context):
-                    return
-                elif lastKey == "Up" and self._rewindSayAll(context):
-                    return
-
-            self._inSayAll = False
-            self._sayAllContexts = []
-            if not self._lastCommandWasStructNav:
-                text.setCaretOffset(offset)
-            orca.emitRegionChanged(obj, offset)
-            return
-
-        # SayAllContext.COMPLETED doesn't necessarily mean done with SayAll;
-        # just done with the current object. If we're still in SayAll, we do
-        # not want to set the caret (and hence set focus) in a link we just
-        # passed by.
-        try:
-            hypertext = obj.queryHypertext()
-        except NotImplementedError:
-            pass
-        else:
-            linkCount = hypertext.getNLinks()
-            links = [hypertext.getLink(x) for x in range(linkCount)]
-            if [l for l in links if l.startIndex <= offset <= l.endIndex]:
-                return
-
-        orca.emitRegionChanged(obj, offset, mode=orca.SAY_ALL)
-        text.setCaretOffset(offset)
-
-    def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
-        """To-be-removed. Returns the string, caretOffset, startOffset."""
-
-        textLine = super().getTextLineAtCaret(obj, offset, startOffset, endOffset)
-        string = textLine[0]
-        if string and string.find(self.EMBEDDED_OBJECT_CHARACTER) == -1 \
-           and obj.getState().contains(pyatspi.STATE_FOCUSED):
-            return textLine
-
-        textLine[0] = self.utilities.displayedText(obj)
-        try:
-            text = obj.queryText()
-        except:
-            pass
-        else:
-            textLine[1] = min(textLine[1], text.characterCount)
-
-        return textLine
-
     def updateBraille(self, obj, **args):
         """Updates the braille display to show the given object.
 
@@ -629,9 +449,9 @@ class Script(default.Script):
         - obj: the Accessible
         """
 
-        if not _settingsManager.getSetting('enableBraille') \
-           and not _settingsManager.getSetting('enableBrailleMonitor'):
-            debug.println(debug.LEVEL_INFO, "BRAILLE: update disabled", True)
+        if not settings_manager.getManager().getSetting('enableBraille') \
+           and not settings_manager.getManager().getSetting('enableBrailleMonitor'):
+            debug.printMessage(debug.LEVEL_INFO, "BRAILLE: update disabled", True)
             return
 
         if not obj:
@@ -644,15 +464,15 @@ class Script(default.Script):
             return
 
         brailleLine = self.getNewBrailleLine(clearBraille=True, addLine=True)
-        for child in obj:
-            if not self.utilities.onSameLine(child, obj[0]):
+        for child in AXObject.iter_children(obj):
+            if not AXComponent.on_same_line(child, AXObject.get_child(obj, 0)):
                 break
             [regions, fRegion] = self.brailleGenerator.generateBraille(child)
             self.addBrailleRegionsToLine(regions, brailleLine)
 
         if not brailleLine.regions:
             [regions, fRegion] = self.brailleGenerator.generateBraille(
-                obj, role=pyatspi.ROLE_PARAGRAPH)
+                obj, role=Atspi.Role.PARAGRAPH)
             self.addBrailleRegionsToLine(regions, brailleLine)
             self.setBrailleFocus(fRegion)
 
